Package: Dino
Type: Package
Title: Normalization of Single-Cell mRNA Sequencing Data
Version: 1.18.0
Authors@R: c(
    person("Jared", "Brown", email = "brownj@ds.dfci.harvard.edu",
    role = c("aut", "cre"), comment = c(ORCID = "0000-0002-9151-4386")),
    person("Christina", "Kendziorski", role= c("ctb")))
biocViews: Software, Normalization, RNASeq, SingleCell, Sequencing,
        GeneExpression, Transcriptomics, Regression, CellBasedAssays
Description: Dino normalizes single-cell, mRNA sequencing data to
        correct for technical variation, particularly sequencing depth,
        prior to downstream analysis. The approach produces a matrix of
        corrected expression for which the dependency between
        sequencing depth and the full distribution of normalized
        expression; many existing methods aim to remove only the
        dependency between sequencing depth and the mean of the
        normalized expression. This is particuarly useful in the
        context of highly sparse datasets such as those produced by 10X
        genomics and other uninque molecular identifier (UMI) based
        microfluidics protocols for which the depth-dependent
        proportion of zeros in the raw expression data can otherwise
        present a challenge.
Depends: R (>= 4.0.0)
License: GPL-3
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.1.1
Suggests: testthat (>= 2.1.0), knitr, rmarkdown, BiocStyle, devtools,
        ggplot2, gridExtra, ggpubr, grid, magick, hexbin
VignetteBuilder: knitr
Imports: BiocParallel, BiocSingular, SummarizedExperiment,
        SingleCellExperiment, S4Vectors, Matrix, Seurat, matrixStats,
        parallel, scran, grDevices, stats, methods
URL: https://github.com/JBrownBiostat/Dino
BugReports: https://github.com/JBrownBiostat/Dino/issues
Config/pak/sysreqs: cmake libglpk-dev make libicu-dev libpng-dev
        libuv1-dev libxml2-dev libssl-dev python3 zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:56:29 UTC
RemoteUrl: https://github.com/bioc/Dino
RemoteRef: RELEASE_3_23
RemoteSha: a4aa74c9092bacb6dac689f888ae3669cbd73b0e
NeedsCompilation: no
Packaged: 2026-04-29 21:01:53 UTC; root
Author: Jared Brown [aut, cre] (ORCID: <https://orcid.org/0000-0002-9151-4386>),
  Christina Kendziorski [ctb]
Maintainer: Jared Brown <brownj@ds.dfci.harvard.edu>
Built: R 4.6.0; ; 2026-04-29 21:46:20 UTC; unix
