# Generated by roxygen2: do not edit by hand

export(Dino)
export(Dino_SCE)
export(SeuratFromDino)
import(SingleCellExperiment)
importClassesFrom(Matrix,dgCMatrix)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,register)
importFrom(BiocSingular,IrlbaParam)
importFrom(Matrix,Matrix)
importFrom(Matrix,colSums)
importFrom(Matrix,rowSums)
importFrom(Matrix,t)
importFrom(S4Vectors,metadata)
importFrom(Seurat,CreateAssayObject)
importFrom(Seurat,CreateSeuratObject)
importFrom(Seurat,Misc)
importFrom(SummarizedExperiment,assayNames)
importFrom(grDevices,chull)
importFrom(matrixStats,colMaxs)
importFrom(matrixStats,rowMaxs)
importFrom(methods,is)
importFrom(methods,new)
importFrom(parallel,detectCores)
importFrom(scran,quickCluster)
importFrom(stats,approxfun)
importFrom(stats,bw.nrd)
importFrom(stats,bw.ucv)
importFrom(stats,density)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,optimize)
importFrom(stats,pgamma)
importFrom(stats,ppois)
importFrom(stats,qgamma)
importFrom(stats,rgamma)
importFrom(stats,runif)
