#' Plot data from simulated expression
#'
#' This data is used in the vignette to demonstrate the flexibility of the
#' Dino model to smoothly estimate arbitrary latent uninmodal expression
#' distributions. These data are intended for internal use only.
#'
#' @docType data
#'
#' @usage data("unimodalDat")
#'
#' @format Object of class \code{"gtable"}.
#'
#' @keywords datasets
#'
#' @examples
#' data("unimodalDat")
#'
"unimodalDat"


#' Plot data from simulated expression
#'
#' This data is used in the vignette to demonstrate the flexibility of the
#' Dino model to smoothly estimate arbitrary latent multimodal expression
#' distributions. These data are intended for internal use only.
#'
#' @docType data
#'
#' @usage data("multimodalDat")
#'
#' @format Object of class \code{"gtable"}.
#'
#' @keywords datasets
#'
#' @examples
#' data("multimodalDat")
#'
"multimodalDat"
