% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/postSimPlots.R
\docType{data}
\name{multimodalDat}
\alias{multimodalDat}
\title{Plot data from simulated expression}
\format{
Object of class \code{"gtable"}.
}
\usage{
data("multimodalDat")
}
\description{
This data is used in the vignette to demonstrate the flexibility of the
Dino model to smoothly estimate arbitrary latent multimodal expression
distributions. These data are intended for internal use only.
}
\examples{
data("multimodalDat")

}
\keyword{datasets}
